#ifndef PROXY_H
#define PROXY_H

#include <boost/log/trivial.hpp>
#include <terminal_api/ControlProtoCInterface.h>


std::ostream& operator<<(std::ostream& out, CP_Result result);

namespace api_driver {
    /**
     * Функция-хелпер для логгирования ошибок всех вызовов API.
     * @return то же значение ошибки, что и было передано в качестве параметра
     */
    CP_Result logCpApiError(const char* desc, CP_Result err) {
        if (err != OK) {
            BOOST_LOG_TRIVIAL(error) << "CP API error in " << desc << ": " << err;
        }
        return err;
    }
}

#define CPAPI_PROXY_CALL(proxy, func, ...) do { auto _res = func(proxy.sid, __VA_ARGS__); if (_res != OK) { BOOST_LOG_TRIVIAL(error) << "CP API error in " #func "(" #__VA_ARGS__ "): " << _res; } } while (0)

// void foo() {
//     api_driver::proxy::CpProxy proxy;
//     std::string tmp;
//     CPAPI_PROXY_CALL(proxy, CP_GetDmaDebug, "fuck", &tmp);
// }

namespace api_driver::proxy {
    class CpProxy {
    public:
        TSID sid;
        CpProxy();
        CpProxy(TSID s);

        std::string getDmaDebug(TSID sid, const std::string& arg, const char* calledFrom = "");

        void getModState(TSID sid, modulator_state& dest, const char* calledFrom = "");
        void getModSettings(TSID sid, modulator_settings& dest, const char* calledFrom = "");
        void setModSettings(TSID sid, modulator_settings& dest, const char* calledFrom = "");

        void getDemodState(TSID sid, demodulator_state& dest, const char* calledFrom = "");
        void getDemodSettings(TSID sid, demodulator_settings& dest, const char* calledFrom = "");
        void setDemodSettings(TSID sid, demodulator_settings& dest, const char* calledFrom = "");

#ifdef MODEM_IS_SCPC
        void getCincState(TSID sid, demodulator_settings& dest, const char* calledFrom = "");
#endif

        ~CpProxy();
    };
}


#endif //PROXY_H
